#!/usr/bin/perl

my %BASE;
my %STOP;

my %F;
my %f;
my %G;
my %L;
my %DIC;

open(F,"stopword") || die "cannot open stopword\n";
while(<F>) {
  chomp;
  next if (/^\s*$/);
  next if (/^\S+\s+\S+/);
  $STOP{$_} = 1;
}
close(F);

open(F,"base") || die "cannot open base\n";
while(<F>) {
  chomp;
  my ($a,$b) = split();
  $BASE{$a} = $b;
}
close(F);

my $doc;
my $docnum;
my $term;

while(<>) {
   chomp;
   next if (/^\.W\s*$/ );
   if (/^\.I (\d+)\s*$/) {
      $docnum = $doc = $1;
      next;
   }

   for (split()) {
      for $term (&normalize($_)) {
           next if (! $term);
	   $f{$term}{$doc}++;
	   $F{$term}++;
      }
   }
}

for $term (sort keys %F) {
  next if ($F{$term} <= 1);
  for $doc (1..$docnum) {
      if (defined $f{$term}{$doc}) {
        $G{$term} += ($f{$term}{$doc}/$F{$term} * log($f{$term}{$doc}/$F{$term}));
        $L{$term}{$doc} = 1 + log($f{$term}{$doc});
      }
  }
  $G{$term} = 1 + $G{$term}/log($docnum);
}

my $i = 1;
open (S, ">MED.g");
for $term (sort keys %G) {
     $DIC{$term} = $i;
     printf S "%d %s %.8g\n",$i, $term,$G{$term};
     $i++;
}
close(S);

open (S, ">MED.mat");
for $doc (1..$docnum) {
   print S "$doc";
   for $term (sort keys %F) {
      if (defined $L{$term}{$doc}) {
        printf S " %d:%.8g",$DIC{$term}, $L{$term}{$doc} * $G{$term};
      }
   }
   print S "\n";
}
close(S);

sub normalize
{
   my (@a);
   for (split(/[-\/]/,$_[0])) {
     push(@a, $_);
   }
   
   my $term;
   my @res;
   for $term (@a) {
     next if ($term =~ /\d+/);
     next if ($term =~ /[&\+<=\'`]/);
     next if ($term =~ /^\s*$/);
     
     $term =~ s/^[\"\'\(\)\.,:;\-\/%]//g;
     $term =~ s/^[\"\'\(\)\.,:;\-\/%]//g;	 
     $term =~ s/[\"\'\(\)\.,:;\-\/%]$//g;
     $term =~ s/[\"\'\(\)\.,:;\-\/%]$//g;	 
     $term =~ s/\'s$//g;
   
     if (! defined $STOP{$term}) {
        $term = $BASE{$term} if (defined $BASE{$term});
        push(@res, $term);
     }
  }
  
  return @res;
}
